package melmore.android;

import java.io.FileOutputStream;
import java.io.IOException;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.os.Bundle;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import com.tomgibara.android.camera.CameraSource;
import com.tomgibara.android.camera.SocketCamera;

/*
 * This is a dummy application to display the image on the camera device
 * and allow a user to take pictures and send them to a destination server.
 * 
 * This class and its private class PreviewThread are highly derived from the
 * CameraPreview class in the ApiDemos sample code provided with the Android
 * SDK.
 */

public class DummyCamera extends Activity implements SurfaceHolder.Callback {
	SurfaceHolder  mHolder;
    private boolean mDone;
    private PreviewThread mPreviewThread;
    private boolean mHasSurface;
    private boolean mPreviewMode;
    private Bitmap mTakenPicture;
    private Intent mTranslateIntent;
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        setContentView(R.layout.main);
        
        //set up select intent for later use
        mTranslateIntent = new Intent(this, TranslationClient.class);
        
        //default mode - take a picture
        mPreviewMode = true;
        
        //set up bitmap to hold current camera image
		mTakenPicture = Bitmap.createBitmap(320, 240, false);
        
        //set up buttons
        //mode button - preview mode (take a picture) or still mode (have taken a picture)
        Button modeButton = (Button) findViewById(R.id.modebutton);
        modeButton.setText(R.string.take);
        modeButton.requestFocus();
        
      	modeButton.setOnClickListener(new View.OnClickListener() {
      		public void onClick(View view) {
      			//if we're in preview mode, take a still from the camera preview
      			if (mPreviewMode) {
      				mPreviewMode = false;
      				
      				//stop the updates of the camera
      				if (mPreviewThread != null) {
      		            mPreviewThread.requestExitAndWait();
      		            mPreviewThread = null;
      		        }
      				
      				//write the bitmap out so Megan can look at it :/
      				FileOutputStream fos;
      				try {
      					fos = view.getContext().openFileOutput("cameraimage.jpg", 0);
      					Bitmap b = mTakenPicture;
      					boolean took = mTakenPicture.compress(Bitmap.CompressFormat.JPEG, 100, fos);
      					if (!took) {
      						System.err.println("It didn't take.");
      					}
      					fos.close();
      				}
      				catch (IOException ie) {
      					System.err.println(ie);
      				}
      			
      				//change the mode button text
      				((Button)view).setText(R.string.return_prev);
      				
      				//make the send button visible
      				((Button)findViewById(R.id.translatebutton)).setVisibility(View.VISIBLE);
      				
      				//set the status string
      				((TextView)findViewById(R.id.status)).setText(R.string.taken_status);
      			}
      			
      			//if we're in still mode, return to preview mode
      			else {
      				mPreviewMode = true;
      				
      				//start the updates of the camera
      				if (mPreviewThread == null) {
      		            mPreviewThread = new PreviewThread();
      		            // If we already have a surface, just start the thread now too.
      		            if (mHasSurface == true) {
      		                mPreviewThread.start();
      		            }
      		        }
      				
      				//change the mode button text
      				((Button)view).setText(R.string.take);
      				
      				//make the send button invisible
      				((Button)findViewById(R.id.translatebutton)).setVisibility(View.INVISIBLE);
      				
      				//set the status string
      				((TextView)findViewById(R.id.status)).setText(R.string.default_status);
      			}
      		}
      	});
        
      	//send button - start the TranslationClient activity
        Button translateButton = (Button) findViewById(R.id.translatebutton);
        translateButton.setVisibility(View.INVISIBLE);
        
        translateButton.setOnClickListener(new View.OnClickListener() {
        	public void onClick(View view) {
        		
        		//start the TextSelector activity
        		mTranslateIntent.putExtra("melmore.android.camera_image", mTakenPicture);
        		startActivity(mTranslateIntent);
        	}
        });
        
        //set up status
        TextView status = (TextView) findViewById(R.id.status);
        status.setText(R.string.default_status);
        
        //set up camera display surface
        mDone = false;
        
        SurfaceView cameraView = (SurfaceView) findViewById(R.id.cameraview);
        mHolder = cameraView.getHolder();
        mHolder.setFixedSize(320, 240);
        
        mHolder.addCallback(this);
        mHasSurface = false;
    }
    
    @Override
	protected void onResume() {
        // Because the CameraDevice object is not a shared resource,
        // it's very important to release it when the activity is paused.
        super.onResume();
     
        // We do the actual acquisition in a separate thread. Create it now, if we're in preview mode
        if (mPreviewThread == null && mPreviewMode) {
            mPreviewThread = new PreviewThread();
            // If we already have a surface, just start the thread now too.
            if (mHasSurface == true) {
                mPreviewThread.start();
            }
        }
    }

    @Override
	protected void onPause() {
        // Start Preview again when we resume.
        super.onPause();
     
        // Stop Preview.
        if (mPreviewThread != null) {
            mPreviewThread.requestExitAndWait();
            mPreviewThread = null;
        }
    }
    
    public void surfaceCreated(SurfaceHolder holder) {
        // The Surface has been created, start our main acquisition thread.
        mHasSurface = true;
        if (mPreviewThread != null && mPreviewMode) {
            mPreviewThread.start();
        }
        
        //if we're not in preview mode, refresh the image
        if (!mPreviewMode) {
        	Canvas tempCanvas = mHolder.lockCanvas();
        	tempCanvas.drawBitmap(mTakenPicture, 0, 0, new Paint());
        	mHolder.unlockCanvasAndPost(tempCanvas);
        }
    }

    public void surfaceDestroyed(SurfaceHolder holder) {
        // Surface will be destroyed when we return. Stop the preview.
        mHasSurface = false;
    }

    public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) {
        // Surface size or format has changed. This should not happen in this
        // example.
    }
    
    
    class PreviewThread extends Thread {
        private boolean mDone;
        private CameraSource mCamera;
        
        PreviewThread() {
            super();
            mDone = false;
        }
    
        @Override
		public void run() {
            // We first open the socket camera - no configuration needed.
            mCamera = new SocketCamera("128.112.48.181", 9889, 320, 240, true);
            if (!mCamera.open()) {
            	//don't capture from it
            	mCamera = null;
            	mDone = true;
            }

            // This is our main acquisition thread's loop, we go until
            // asked to quit.
            SurfaceHolder holder = mHolder;
            while (!mDone) {
                // Lock the surface, this returns a Canvas that can
                // be used to render into.
                Canvas canvas = holder.lockCanvas();

                // Capture directly into the Surface
                if (mCamera != null) {
                    mCamera.capture(canvas);
                }
                
                // And finally unlock and post the surface.
                holder.unlockCanvasAndPost(canvas);
            }

            // Make sure to release the CameraDevice
            if (mCamera != null)
                mCamera.close();
        }
        
        public void requestExitAndWait() {
            // don't call this from PreviewThread thread or it a guaranteed
            // deadlock!
            
            Canvas tempCanvas;
            
            //save the current picture as a bitmap
            if (mCamera != null) {
            	tempCanvas = new Canvas(mTakenPicture);
            	mCamera.capture(tempCanvas);
            	Bitmap b = mTakenPicture;
            }
            
            mDone = true;
            
            try {
                join();
            } catch (InterruptedException ex) { }
        }
    }
}