package melmore.android;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Point;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

public class TextSelector extends Activity implements SurfaceHolder.Callback {
	private Bitmap mCameraImage;
	private Bitmap mImageSelection;
	private Canvas mEditDisplay;
	private SurfaceHolder mHolder;
	private boolean mCorner1Selected;
	private boolean mCorner2Selected;
	private boolean mHasSurface;
	private boolean mInReviewView;
	private Point mCorner1Center;
	private Point mCorner2Center;
	private Intent mTranslateIntent;

	/** Called when the activity is first created. */
    @Override
	public void onCreate(Bundle icicle) {
    	super.onCreate(icicle);
        setContentView(R.layout.selector);
        
        //set up translate intent for later use
        mTranslateIntent = new Intent(this, TranslationClient.class);
        
        //we haven't selected any corners yet
        mCorner1Selected = false;
        mCorner2Selected = false;
        mInReviewView = false;
        
        //pull out the bitmap we want to draw
        mCameraImage = (Bitmap)this.getIntent().getParcelableExtra("melmore.android.camera_image");
        
        //set up view, but don't try to draw on view until we have a surface
        SurfaceView editView = (SurfaceView) findViewById(R.id.editview);
	    mHolder = editView.getHolder();
	    mHolder.setFixedSize(320, 240);
	    mHolder.addCallback(this);
	    
        mHasSurface = false;
        
        //make all buttons invisible
        ((Button)findViewById(R.id.sendbutton)).setVisibility(View.INVISIBLE);
        ((Button)findViewById(R.id.reselectbutton)).setVisibility(View.INVISIBLE);
        ((Button)findViewById(R.id.returnbutton)).setVisibility(View.INVISIBLE);
        
        //set first help text
        ((TextView)findViewById(R.id.editstatus)).setText(R.string.select_first_corner);
	}
    
    @Override
    public void onResume() {
    	super.onResume();
    }
	
    public void surfaceCreated(SurfaceHolder holder) {
        // The Surface has been created
        mHasSurface = true;
        
	    //draw the edit view
        if (mInReviewView) {
        	enterReviewView();
        }
        else {
        	// set up the first corner marker
            mCorner1Center = new Point(160, 120);
            
        	drawEditView();
        }
    }

    public void surfaceDestroyed(SurfaceHolder holder) {
        // Surface will be destroyed when we return. Stop editing.
        mHasSurface = false;
    }

    public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) {
        // Surface size or format has changed. This should not happen in this
        // example.
    }
    
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
    	
    	//handle selection events if both corners have not been selected yet
    	if (!mCorner2Selected) {
    		if (keyCode == KeyEvent.KEYCODE_DPAD_LEFT) {
    			if (!mCorner1Selected) {
    				if (mCorner1Center.x - 2 >= 0) {
    					mCorner1Center.set(mCorner1Center.x - 1, mCorner1Center.y);
    				}
    			}
    			else {
    				if (mCorner2Center.x - 2 >= 0) {
    					mCorner2Center.set(mCorner2Center.x - 1, mCorner2Center.y);
    				}
    			}
    			drawEditView();
    		}
    		if (keyCode == KeyEvent.KEYCODE_DPAD_RIGHT) {
    			if (!mCorner1Selected) {
    				if (mCorner1Center.x + 2 <= 319) {
    					mCorner1Center.set(mCorner1Center.x + 1, mCorner1Center.y);
    				}
    			}
    			else {
    				if (mCorner2Center.x + 2 <= 319) {
    					mCorner2Center.set(mCorner2Center.x + 1, mCorner2Center.y);
    				}
    			}
    			drawEditView();
    		}
			if (keyCode == KeyEvent.KEYCODE_DPAD_UP) {
    			if (!mCorner1Selected) {
    				if (mCorner1Center.y - 2 >= 0) {
    					mCorner1Center.set(mCorner1Center.x, mCorner1Center.y - 1);
    				}
    			}
    			else {
    				if (mCorner2Center.y - 2 >= 0) {
    					mCorner2Center.set(mCorner2Center.x, mCorner2Center.y - 1);
    				}
    			}
    			drawEditView();
			}
			if (keyCode == KeyEvent.KEYCODE_DPAD_DOWN) {
    			if (!mCorner1Selected) {
    				if (mCorner1Center.y + 2 <= 239) {
    					mCorner1Center.set(mCorner1Center.x, mCorner1Center.y + 1);
    				}
    			}
    			else {
    				if (mCorner2Center.y + 2 <= 239) {
    					mCorner2Center.set(mCorner2Center.x, mCorner2Center.y + 1);
    				}
    			}
    			drawEditView();
			}
			if (keyCode == KeyEvent.KEYCODE_DPAD_CENTER) {
    			if (!mCorner1Selected) {
    				
    				//fix first corner and set second corner to be incident with first
    				mCorner1Selected = true;
    				mCorner2Center = new Point(mCorner1Center.x, mCorner1Center.y);
    				
    				//set help text for finding second corner
    				((TextView)findViewById(R.id.editstatus)).setText(R.string.select_second_corner);
    				
    				drawEditView();
    			}
    			else {
    				mCorner2Selected = true;
    				
    				//set up review view - will ask user if they like this,
    				//want to try again, or want to return
    				mInReviewView = true;
    				enterReviewView();
    			}
			}
    		return true;
    	}
    	
    	//handle buttons normally if both corners have been selected
    	else {
    		return super.onKeyDown(keyCode, event);
    	}
    }
    
    private void drawEditView() {
    	
    	if (mHasSurface) {
	    	mEditDisplay = mHolder.lockCanvas();
	    	
	    	Paint p = new Paint();
	    	
	    	mEditDisplay.drawBitmap(mCameraImage, 0, 0, p);
	    	
	    	//draw the first corner marker in green if it's not selected
	    	if (!mCorner1Selected) {
	    		p.setColor(Color.GREEN);
	    		mEditDisplay.drawRect(mCorner1Center.x - 2, mCorner1Center.y - 2,
	    				              mCorner1Center.x + 2, mCorner1Center.y + 2, p);
	    		p.setColor(Color.BLACK);
	    		p.setStyle(Paint.Style.STROKE);
	    		p.setStrokeWidth(0);
	    		mEditDisplay.drawRect(mCorner1Center.x - 2, mCorner1Center.y - 2,
	    				              mCorner1Center.x + 2, mCorner1Center.y + 2, p);
	    	}
	    	
	    	//draw the first corner marker in red if it is selected
	    	else {
	    		p.setColor(Color.RED);
	    		mEditDisplay.drawRect(mCorner1Center.x - 2, mCorner1Center.y - 2,
	    				              mCorner1Center.x + 2, mCorner1Center.y + 2, p);
	    		p.setColor(Color.BLACK);
	    		p.setStyle(Paint.Style.STROKE);
	    		p.setStrokeWidth(0);
	    		mEditDisplay.drawRect(mCorner1Center.x - 2, mCorner1Center.y - 2,
	    				              mCorner1Center.x + 2, mCorner1Center.y + 2, p);
	    		
	    		//draw the second corner marker in green
	    		p.setColor(Color.GREEN);
	    		p.setStyle(Paint.Style.FILL);
	    		mEditDisplay.drawRect(mCorner2Center.x - 2, mCorner2Center.y - 2,
	    				              mCorner2Center.x + 2, mCorner2Center.y + 2, p);
	    		p.setColor(Color.BLACK);
	    		p.setStyle(Paint.Style.STROKE);
	    		p.setStrokeWidth(0);
	    		mEditDisplay.drawRect(mCorner2Center.x - 2, mCorner2Center.y - 2,
	    				              mCorner2Center.x + 2, mCorner2Center.y + 2, p);
	    		
	    		//draw the rectangle around these
	    		if (mCorner1Center.x < mCorner2Center.x) {
	    			
	    			//1 top left, 2 bottom right
	    			if (mCorner1Center.y < mCorner2Center.y) {
	    				mEditDisplay.drawRect(mCorner1Center.x - 2, mCorner1Center.y - 2,
	    									  mCorner2Center.x + 2, mCorner2Center.y + 2, p);
	    			}
	    			
	    			//1 bottom left, 2 top right
	    			else {
	    				mEditDisplay.drawRect(mCorner1Center.x - 2, mCorner2Center.y - 2,
								              mCorner2Center.x + 2, mCorner1Center.y + 2, p);
	    			}
	    		}
	    		else {
	    			//1 top right, 2 bottom left
	    			if (mCorner1Center.y < mCorner2Center.y) {
	    				mEditDisplay.drawRect(mCorner2Center.x - 2, mCorner1Center.y - 2,
	    									  mCorner1Center.x + 2, mCorner2Center.y + 2, p);
	    			}
	    			
	    			//1 bottom right, 2 top left
	    			else {
	    				mEditDisplay.drawRect(mCorner2Center.x - 2, mCorner2Center.y - 2,
								              mCorner1Center.x + 2, mCorner1Center.y + 2, p);
	    			}
	    		}
	    	}
	    	
	    	//post the canvas
	    	mHolder.unlockCanvasAndPost(mEditDisplay);
    	}
    }
    
    private void enterReviewView() {
    	
    	//display only the selected region of the image
    	//have to figure out dimensions first, blehhh
    	mImageSelection = null;
    	int selHeight = 0;
    	int selWidth = 0;
    	if (mCorner1Center.x < mCorner2Center.x) {
			
			//1 top left, 2 bottom right
			if (mCorner1Center.y < mCorner2Center.y) {
				selHeight = mCorner2Center.y - mCorner1Center.y + 4;
				selWidth = mCorner2Center.x - mCorner1Center.x + 4;
				mImageSelection = Bitmap.createBitmap(mCameraImage, mCorner1Center.x - 2, 
													 mCorner1Center.y - 2, 
													 selWidth,
													 selHeight);
			}
			
			//1 bottom left, 2 top right
			else {
				selHeight = mCorner1Center.y - mCorner2Center.y + 4;
				selWidth = mCorner2Center.x - mCorner1Center.x + 4;
				mImageSelection = Bitmap.createBitmap(mCameraImage, mCorner1Center.x - 2, 
						 							 mCorner2Center.y - 2, 
						 							 selWidth,
						 							 selHeight);
			}
		}
		else {
			//1 top right, 2 bottom left
			if (mCorner1Center.y < mCorner2Center.y) {
				selHeight = mCorner2Center.y - mCorner1Center.y + 4;
				selWidth = mCorner1Center.x - mCorner2Center.x + 4;
				mImageSelection = Bitmap.createBitmap(mCameraImage, mCorner1Center.x - 2, 
						 							 mCorner2Center.y - 2, 
						 							 selWidth,
						 							 selHeight);
			}
			
			//1 bottom right, 2 top left
			else {
				selHeight = mCorner1Center.y - mCorner2Center.y + 4;
				selWidth = mCorner1Center.x - mCorner2Center.x + 4;
				mImageSelection = Bitmap.createBitmap(mCameraImage, mCorner2Center.x - 2, 
						 							 mCorner2Center.y - 2, 
						 							 selWidth,
						 							 selHeight);
			}
		}
    	
    	mEditDisplay = mHolder.lockCanvas();
    	Paint p = new Paint();
    	p.setColor(Color.WHITE);
    	mEditDisplay.drawRect(0, 0, 319, 239, p);
    	mEditDisplay.drawBitmap(mImageSelection, 160 - (selWidth / 2), 120 - (selHeight / 2), p);
    	mHolder.unlockCanvasAndPost(mEditDisplay);
    	
        //set help text
        ((TextView)findViewById(R.id.editstatus)).setText(R.string.post_selection);
    	
    	//set all buttons to be visible
        ((Button)findViewById(R.id.sendbutton)).setVisibility(View.VISIBLE);
        ((Button)findViewById(R.id.reselectbutton)).setVisibility(View.VISIBLE);
        ((Button)findViewById(R.id.returnbutton)).setVisibility(View.VISIBLE);
        
        ((Button)findViewById(R.id.sendbutton)).requestFocus();
        
        //handle trying to send - start new send activity
        ((Button)findViewById(R.id.sendbutton)).setOnClickListener(new View.OnClickListener() {
        	public void onClick(View view) {
        		
        		mTranslateIntent.putExtra("melmore.android.selection_image",
        									((TextSelector)(view.getContext())).mImageSelection);
        		startActivity(mTranslateIntent);
        		
        	}
        });
        
        //handle reselection - go back to selection mode
        ((Button)findViewById(R.id.reselectbutton)).setOnClickListener(new View.OnClickListener() {
        	public void onClick(View view) {
        		
        		//no longer in review view
        		mInReviewView = false;
        		
        		//no corners have been selected
        		mCorner1Selected = false;
        		mCorner2Selected = false;
        		
        		//set all buttons to be invisible
                ((Button)findViewById(R.id.sendbutton)).setVisibility(View.INVISIBLE);
                ((Button)findViewById(R.id.reselectbutton)).setVisibility(View.INVISIBLE);
                ((Button)findViewById(R.id.returnbutton)).setVisibility(View.INVISIBLE);
                
                //set first help text
                ((TextView)findViewById(R.id.editstatus)).setText(R.string.select_first_corner);
        		
        		//draw the edit view
        		((TextSelector)view.getContext()).drawEditView();
        	}
        });
        
        //handle return - finish this activity and go back
        ((Button)findViewById(R.id.returnbutton)).setOnClickListener(new View.OnClickListener() {
        	public void onClick(View view) {
        		((Activity)view.getContext()).finish();
        	}
        });
    }
}
